;;; Sierra Script 1.0 - (do not remove this comment)
;
; SCI Template Game
; By Brian Provinciano
; ******************************************************************************
; main.sc
; Contains the game's main instance and inventory items.
(script# MAIN)
(include game.sh)
(use intrface)
(use sound)
(use save)
(use motion)
(use game)
(use File)
(use user)
(use menu)
(use actor)
(use system)

(public
	Tetris 0
	GameOver 2
)

(local
	ego									;pointer to ego
	theGame								;ID of the Game instance
	curRoom								;ID of current room
	speed =  6							;number of ticks between animations
	quit								;when TRUE, quit game
	cast								;collection of actors
	regions								;set of current regions
	timers								;list of timers in the game
	sounds								;set of sounds being played
	inventory							;set of inventory items in game
	addToPics							;list of views added to the picture
	curRoomNum							;current room number
	prevRoomNum							;previous room number
	newRoomNum							;number of room to change to
	debugOn								;generic debug flag -- set from debug menu
	score								;the player's current score
	possibleScore						;highest possible score
	showStyle	=		IRISOUT			;style of picture showing
	aniInterval							;# of ticks it took to do the last animation cycle
	theCursor							;the number of the current cursor
	normalCursor =		ARROW_CURSOR	;number of normal cursor form
	waitCursor	 =		HAND_CURSOR		;cursor number of "wait" cursor
	userFont	 =		USERFONT		;font to use for Print
	smallFont	 =		4				;small font for save/restore, etc.
	lastEvent							;the last event (used by save/restore game)
	modelessDialog						;the modeless Dialog known to User and Intrface
	bigFont		=		USERFONT		;large font
	volume		=		12				;sound volume
	version		=		{x.yyy.zzz}		;pointer to 'incver' version string			
	locales								;set of current locales
	[curSaveDir 20]						;address of current save drive/directory string
	aniThreshold	=	10
	perspective							;player's viewing angle:
										;	 degrees away from vertical along y axis
	features							;locations that may respond to events
	sortedFeatures          			;above+cast sorted by "visibility" to ego
	useSortedFeatures					;enable cast & feature sorting?
	demoScripts							;add to curRoomNum to find room demo script
	egoBlindSpot						;used by sortCopy to exclude
										;actors behind ego within angle 
										;from straight behind. 
										;Default zero is no blind spot
	overlays	=		-1
	doMotionCue							;a motion cue has occurred - process it
	systemWindow						;ID of standard system window
	demoDialogTime	=	3				;how long Prints stay up in demo mode
	currentPalette
	modelessPort
	;globals 63-99 are unused
		global63
		global64
		global65
		global66
		global67
		global68
		global69
		global70
		global71
		global72
		global73
		global74
		global75
		global76
		global77
		global78
		global79
		global80
		global81
		global82
		global83
		global84
		global85
		global86
		global87
		global88
		global89
		global90
		global91
		global92
		global93
		global94
		global95
		global96
		global97
		global98
	lastSysGlobal
	;globals 100 and above are for game use
	music
	gLevel = 1
)

(instance Tetris of Game

	(method (init &tmp [str 40])
		; Initialize
		(= systemWindow SysWindow)
		(super init:)
;
;         * Set your game version here *
		(= version {1.1 (Oct 2020)})
		; General initialization stuff
		(StatusLine code: statusCode)
		(TheMenuBar init:)
		((= music theMusic)
			owner: self
			init:
		)
		
		;set the high score
		(if (scoreFile open: fRead)
			(scoreFile read: str 40)
			(= possibleScore (ReadNumber str))
		) 
		(scoreFile close:)
			
		(= ego egoObj)
		(User alterEgo: ego blocks: 0 y: 150)
		(if (HaveMouse)
			(theGame setCursor: normalCursor TRUE)
		else
			(theGame
				setCursor: normalCursor TRUE 304 174
			)
		)
		; Start the room
		(self newRoom: TITLE)
	)
)

(instance statusCode of Code

	(method (doit strg)
		(Format strg
			"___________________________TETRIS____________Level: %d"
			gLevel
		)
	)
)


(instance egoObj of Ego
	(properties
		name {ego}
	)
)

(instance theMusic of Sound
	(properties
		number 1
	)
)

(instance scoreFile of File
	(properties
		name "hiscore"
	)
)

;
; * THE PUBLIC PROCEDURES

(procedure (hiscore &tmp [str 40])
	(if (> score possibleScore)
		(= possibleScore score)
		(if (scoreFile open: fTrunc)
			(Format @str "%d" possibleScore)
			(scoreFile write: @str)
		)
		(scoreFile close:)	
		(return TRUE)
	)
	(return FALSE)
)

(procedure (GameOver)
	(if (hiscore)
		(Printf
			"Congratulations! You got a new High Score!\n\n
			Your Score: %d" possibleScore
		)
	)
	(repeat
		(switch
			;; pick a random russian saying
			(Print 100 (Random 0 27)
				#title {GAME OVER}
				#icon 800 0 0
				#button {Restart} 1
				#button {Quit} 2
			)
			(1
				(theGame restart:)
			)
			(2
				(= quit TRUE)
				(break)
			)
		)
	)
)