;;; Sierra Script 1.0 - (do not remove this comment)
;
; SCI Template Game
; By Brian Provinciano
; ******************************************************************************
; menubar.sc
; Contains the customized Menubar class. This is the script you modify if you 
; want to customize the menu.
(script# MENU)
(include game.sh) (include menu.sh)
(use main)
(use intrface)
(use gauge)
(use sound)
(use user)

(public
	ToggleSound 0
)




(class TheMenuBar of MenuBar
	(properties
		state 0
	)
	
	(method (init)
		(AddMenu { \3F_}
			{About this Game`^a:Help`#1}
		)
		(AddMenu { File_}
			{Restart Game`#9:Pause Game`^p:--! :Quit`^q}
		)
		(if (DoSound SoundOn)
			(AddMenu { Sound_}
				{Volume...`^v:Turn Off`#2=1}
			)
		else
			(AddMenu { Sound_}
				{Volume...`^v:Turn On`#2=1}
			)
		)
	)
	
	(method (handleEvent event &tmp i oldPause [tempStr 40])
		(switch (super handleEvent: event)
			(aboutI
				(Format @tempStr "Tetris for SCI0 v%s" version)
				(Print
					"Created by Rainer De Temple.\n\n
					Original Tetris by Alexey Pajitnov 1984.\n\n
					Template Game By Brian Provinciano.\n\n
					Created with SCICompanion by Phil Fortier et al.
					\n\nSierra's Creative Interpreter by Jeff Stephenson et al."
					@tempStr
					#font 999
				)
			)
			(helpI
				(Print
					"Stack the Tetrominoes to single lines at the bottom of the play area.\n
					Bonus points for multiple lines.\n\n
					Use the WASD or arrow keys to control.\n\n
					ESC brings up the menu.\n\n
					F9 to Restart Game.\n\n
					Control-Q to quit."
					#title {How To Play}
				)
			)
			(restartI
				(if
					(Print
						"Are you serious? You really want to start all the way back at the beginning again?"
						#title {Restart}
						#button {Restart} 1
						#button { Oops_} 0
					)
					(theGame restart:)
				)
			)
			(quitI
				(= quit
					(Print
						"Do you really want to quit?"
						#title {Quit}
						#button { Quit_} 1
						#button { Oops_} 0
					)
				)
			)
			(pauseI
				(= oldPause (Sound pause:))
				;; pick a random russian saying
				(Print 100 (Random 0 27)
					#title	{PAUSE}
					#button {Ok. I'm back.})				
				(Sound pause: oldPause)
			)
			(volumeI
				(= i
					((Gauge new:)
						text: {Sound Volume}
						description:
							{Use the mouse or the left and right arrow keys to adjust the volume.}
						higher: {Louder}
						lower: {Softer}
						normal: 15
						doit: (DoSound ChangeVolume i)
					)
				)
				(DoSound ChangeVolume i)
				(DisposeScript GAUGE)
			)
			(soundI
				(ToggleSound)
			)
		)
	)
)


(procedure (ToggleSound &tmp SOUND_OFF)
	(= SOUND_OFF (DoSound SoundOn))
	(= SOUND_OFF (DoSound SoundOn (not SOUND_OFF)))
	(if SOUND_OFF
		(SetMenu soundI p_text {Turn On})
	else
		(SetMenu soundI p_text {Turn Off})
	)
)
